@@ -1,5 +1,6 @@
 import copy
 from dataclasses import is_dataclass
+from itertools import zip_longest
 from typing import TypeVar, Type, Optional, get_type_hints, Mapping, Any
 
 from dacite.config import Config
@@ -13,6 +14,7 @@ from dacite.exceptions import (
     MissingValueError,
     DaciteFieldError,
     UnexpectedDataError,
+    StrictUnionMatchError,
 )
 from dacite.types import (
     is_instance,
@@ -22,6 +24,8 @@ from dacite.types import (
     is_optional,
     transform_value,
     extract_origin_collection,
+    is_init_var,
+    extract_init_var,
 )
 
 T = TypeVar("T")
@@ -78,6 +82,8 @@ def from_dict(data_class: Type[T], data: Data, config: Optional[Config] = None)
 
 
 def _build_value(type_: Type, data: Any, config: Config) -> Any:
+    if is_init_var(type_):
+        type_ = extract_init_var(type_)
     if is_union(type_):
         return _build_value_for_union(union=type_, data=data, config=config)
     elif is_generic_collection(type_) and is_instance(data, extract_origin_collection(type_)):
@@ -91,24 +97,44 @@ def _build_value_for_union(union: Type, data: Any, config: Config) -> Any:
     types = extract_generic(union)
     if is_optional(union) and len(types) == 2:
         return _build_value(type_=types[0], data=data, config=config)
+    union_matches = {}
     for inner_type in types:
         try:
+            # noinspection PyBroadException
+            try:
+                data = transform_value(
+                    type_hooks=config.type_hooks, cast=config.cast, target_type=inner_type, value=data
+                )
+            except Exception:  # pylint: disable=broad-except
+                continue
             value = _build_value(type_=inner_type, data=data, config=config)
             if is_instance(value, inner_type):
-                return transform_value(
-                    type_hooks=config.type_hooks, cast=config.cast, target_type=inner_type, value=value
-                )
+                if config.strict_unions_match:
+                    union_matches[inner_type] = value
+                else:
+                    return value
         except DaciteError:
             pass
+    if config.strict_unions_match:
+        if len(union_matches) > 1:
+            raise StrictUnionMatchError(union_matches)
+        return union_matches.popitem()[1]
     if not config.check_types:
         return data
     raise UnionMatchError(field_type=union, value=data)
 
 
 def _build_value_for_collection(collection: Type, data: Any, config: Config) -> Any:
+    data_type = data.__class__
     if is_instance(data, Mapping):
-        return data.__class__(
-            (key, _build_value(type_=extract_generic(collection)[1], data=value, config=config))
-            for key, value in data.items()
+        item_type = extract_generic(collection, defaults=(Any, Any))[1]
+        return data_type((key, _build_value(type_=item_type, data=value, config=config)) for key, value in data.items())
+    elif is_instance(data, tuple):
+        types = extract_generic(collection)
+        if len(types) == 2 and types[1] == Ellipsis:
+            return data_type(_build_value(type_=types[0], data=item, config=config) for item in data)
+        return data_type(
+            _build_value(type_=type_, data=item, config=config) for item, type_ in zip_longest(data, types)
         )
-    return data.__class__(_build_value(type_=extract_generic(collection)[0], data=item, config=config) for item in data)
+    item_type = extract_generic(collection, defaults=(Any,))[0]
+    return data_type(_build_value(type_=item_type, data=item, config=config) for item in data)
